# -*- coding:utf-8 -*-

import os
from SCLibrary.builtin import ValidatorKeywords
from SCLibrary.builtin import RequesterKeywords
from SCLibrary.builtin import DBKeywords
from SCLibrary.builtin import LogListener
from SCLibrary.builtin import RandomKeywords
from SCLibrary.builtin import SelectKeywords
from robot.libraries.BuiltIn import BuiltIn
from robot.api import logger
from SCLibrary.base import DynamicCore, hook_zh

__version__ = '0.8.1'

class SCLibrary(DynamicCore):

    ROBOT_LIBRARY_SCOPE = 'GLOBAL'
    ROBOT_LIBRARY_VERSION = __version__

    def __init__(self):
        libraries = [ValidatorKeywords(), RequesterKeywords(),
                     DBKeywords(), RandomKeywords(), SelectKeywords()]
        DynamicCore.__init__(self, libraries)
        built_in = BuiltIn()
        if built_in.get_variable_value("${RF_DEBUG}") == True:
            self.ROBOT_LIBRARY_LISTENER = LogListener()
        # 复写robot的unic.py，支持Log打印中文
        # hook_zh()



# -*- coding:utf-8 -*-

import json
from SCLibrary.base import keyword
from robot.utils.dotdict import DotDict

class ValidatorKeywords(object):

    @keyword("Json Should Be Equal")
    def compare_json_pass(self, src, dist):
        """ 比较Json结构是否相同，比较对象可以是Json字符串或Dict, List
        ``src``  待比较的Json字符串或Dict, List
        
        ``dist`` 比较的Json字符串或Dict, List
        """
        if self.compare_json(src, dist) == False:
            raise AssertionError('期望Json相同，但是现在不同') 

    @keyword("Json Should Not Be Equal")
    def compare_json_faild(self, src, dist):
        """ 比较Json结构是否不相同，比较对象可以是Json字符串或Dict, List """
        if self.compare_json(src, dist) == True:
            raise AssertionError('期望Json不同，但是现在相同') 

    # loads:json to dict
    # dumps:dict to json
    def compare_json(self, src, dist):
        """ 比较
        ``selectStatement``  查询语句
        
        ``sansTran`` 是否处于事务
        """
        dict1 = src
        dict2 = dist
        if type(src) != DotDict:
            print('Src Json: ' + src)
            print('Dist Json: ' + dist)

            src = src.replace("\\", '').replace('"{', "{").replace('}"', "}")
            dist = dist.replace("\\", '').replace('"{', "{").replace('}"', "}")

            src = src.lower()
            dist = dist.lower()

            # to dict
            dict1 = json.loads(src)
            dict2 = json.loads(dist)

        # modify dict values
        dict1 = self.modify_keys(dict1)
        dict2 = self.modify_keys(dict2)

        topkey_result = self.comp_topkey(dict1, dict2)

        if topkey_result == True:
            subkey_result = self.comp_subkey(dict1, dict2)
        else:
            subkey_result = False

        return subkey_result

    def comp_subkey(self, dict1, dict2):

        ty1 = type(dict1).__name__
        ty2 = type(dict1).__name__

        if ty1 == ty2 and ty1 == "dict":
            keys = dict1.keys()
            for key in keys:
                value1 = dict1[key]
                value2 = dict2[key]

                if value1 == value2:

                    if value1 != "1":
                        subdict = value1
                        self.comp_subkey(subdict, subdict)
                else:
                    print("Wrong Key: " + key)
                    return False
        elif ty1 == ty2 and ty1 == "list":
            num = len(dict1)
            dict1.sort()
            dict2.sort()
            for i in range(num):
                item = dict1[i]
                self.comp_subkey(item, item)

        elif ty1 != ty2:
            return False

        return True

    def comp_topkey(self, dict1, dict2):

        topkeys1 = []
        keys = dict1.keys()
        for key in keys:
            value = dict1[key]
            if value == "1":
                topkeys1.append(key)

        topkeys2 = []
        keys = dict2.keys()
        for key in keys:
            value = dict2[key]
            if value == "1":
                topkeys2.append(key)

        if topkeys1 == topkeys2:
            return True
        else:
            return False

    def modify_keys(self, dict):

        keys = dict.keys()
        for key in keys:
            value = dict[key]
            ty = type(value).__name__

            if ty == "dict":
                subdict = value
                self.modify_keys(subdict)
            elif ty == "str" or ty == "int":
                dict[key] = "1"
            elif ty == "list":
                while ty == "list":
                    value.sort()
                    num = len(value)
                    for i in range(num):
                        item_dic = value[i]
                        item_keys = item_dic.keys()
                        for item_key in item_keys:
                            subitem = item_dic[item_key]
                            item_ty = type(subitem).__name__
                            if item_ty == "dict":
                                subdict = subitem
                                self.modify_keys(subdict)
                                ty = "dic"
                            elif item_ty == "str" or item_ty == "int":
                                item_dic[item_key] = "1"
                                ty = "int"
                            elif item_ty == "list":
                                ty = 'list'
                                value = item_dic[item_key]
                                value.sort()
                            else:
                                item_dic[item_key] = "1"
                                ty = "int"
            else:
                dict[key] = "1"
                # print "other type: " + ty

        dict = str(dict)
        dict = dict.replace("'", '"')
        dict = eval(dict)
        return dict



# -*- coding:utf-8 -*-

from DatabaseLibrary.connection_manager import ConnectionManager
from DatabaseLibrary.query import Query
from DatabaseLibrary.assertion import Assertion
from SCLibrary.base import keyword
import json


class DBKeywords(ConnectionManager, Query, Assertion):

    @keyword("DB Connect")
    def db_connect_to_mysql(self, db):
        """ 连接数据库
        ``db`` 字典，例：{'db':'mysql','host':'172.17.40.214','port=7003','name=todo','user':'root','pwd':'root'}
        其中db为连接的数据库类型名称，默认mysql,  name为数据库名
        """
        module = db['db'] if 'db' in db else 'pymysql'
        if module.upper() == 'MYSQL':
            module = 'pymysql'
        elif module.upper() == 'POSTGRESQL' or module.upper() == 'PGSQL':
            module = 'psycopg2'
        super(DBKeywords, self).connect_to_database(dbapiModuleName=module, dbName=db['name'],
                                                    dbUsername=db['user'], dbPassword=db['pwd'], dbHost=db['host'], dbPort=db['port'], dbCharset='utf8')

    @keyword("DB Disconnect From Database")
    def db_disconnect_db(self):
        """ 断开所有的数据库连接 """
        super(DBKeywords, self).disconnect_from_database()

    @keyword("DB Query")
    def db_query(self, selectStatement, sansTran=False, returnAsDict=True):
        """ 连接数据库
        ``selectStatement``  select语句

        ``sansTran`` 是否处于事务

        ``returnAsDict`` 结果作为字典返回，默认True
        """
        return super(DBKeywords, self).query(selectStatement, sansTran, returnAsDict)

    @keyword("DB Execute Sql Script")
    def db_execute_sql_script(self, sqlScriptFileName, sansTran=False):
        """ 执行一个SQL脚本文件
        ``sqlScriptFileName``  脚本目录

        ``sansTran`` 是否处于事务
        """
        return super(DBKeywords, self).execute_sql_script(sqlScriptFileName, sansTran)

    @keyword("DB Execute Sql String")
    def db_execute_sql_string(self, sqlString, sansTran=False):
        """ 执行一个SQL
        ``sqlString``  sql

        ``sansTran`` 是否处于事务
        """
        if sqlString.startswith('select') or sqlString.startswith('SELECT'):
            return self.query(sqlString, sansTran, True)

        return super(DBKeywords, self).execute_sql_string(
            sqlString, sansTran)

    @keyword("DB Execute")
    def de_execute(self, sqlString, sansTran=False):
        """ 执行一个SQL
        ``sqlString``  sql

        ``sansTran`` 是否处于事务
        """
        self.db_execute_sql_string(sqlString, sansTran=False)

    @keyword("DB Check If Exists In Database")
    def db_exists_in_db(self, selectStatement, sansTran=False):
        """ 断言查询结果是否存在
        ``selectStatement``  查询语句

        ``sansTran`` 是否处于事务
        """
        super(DBKeywords, self).check_if_exists_in_database(
            selectStatement, sansTran)

    @keyword("DB Check If Not Exists In Database")
    def db_not_exists_in_db(self, selectStatement, sansTran=False):
        """ 断言查询结果是否不存在 """
        super(DBKeywords, self).check_if_not_exists_in_database(
            selectStatement, sansTran)



# -*- coding:utf-8 -*-
import random
from SCLibrary.base import keyword

class RandomKeywords(object):
    @keyword("Random Item Of Array")
    def random_item_of_array(self, arr):
        """ 随机返回数组中的某一项 """
        if type(arr) is not list:
            raise ValueError('类型错误：传入的参数不是一个数组')
        length = len(arr)
        if length == 0:
            raise ValueError('数组为空')
        else:
            return arr[random.randint(0, length - 1)]


    @keyword("Random Boolean")
    def boolean(self):
        """ 随机返回一个布尔值，True 或者 False """
        return True if random.randint(0, 1) == 0 else False

    @keyword("Random Float")
    def float(self, start, end):
        """ 随机返回一个float，参数为最小值和最大值(包含) """
        start = float(start)
        end = float(end)
        return random.uniform(start, end)

    @keyword("Random Int")
    def int(self, start, end):
        """ 随机返回一个int，参数为最小值和最大值(包含) """
        start = int(start)
        end = int(end)
        return random.randint(start, end)

    @keyword("Random Name")
    def name(self):
        """ 随机返回一个中文名 """
        last_name = ['王', '李', '张', '刘', '陈', '杨', '赵', '黄', '周', '吴',
                     '徐', '孙', '胡', '朱', '高', '林', '何', '郭', '马', '罗',
                     '梁', '宋', '郑', '谢', '韩', '唐', '冯', '于', '董', '萧',
                     '程', '曹', '袁', '邓', '许', '傅', '沈', '曾', '彭', '吕',
                     '苏', '卢', '蒋', '蔡', '贾', '丁', '魏', '薛', '叶', '阎',
                     '余', '潘', '杜', '戴', '夏', '锺', '汪', '田', '任', '姜',
                     '范', '方', '石', '姚', '谭', '廖', '邹', '熊', '金', '陆',
                     '郝', '孔', '白', '崔', '康', '毛', '邱', '秦', '江', '史',
                     '顾', '侯', '邵', '孟', '龙', '万', '段', '雷', '钱', '汤',
                     '尹', '黎', '易', '常', '武', '乔', '贺', '赖', '龚', '文']
    
        first_name = ['伟', '芳', '娜', '秀英', '敏', '静', '丽', '强', '磊', '军',
                      '洋', '勇', '艳', '杰', '娟', '涛', '明', '超', '秀兰', '霞',
                      '平', '刚', '桂英']
    
        return random.choice(last_name) + random.choice(first_name)
    
    @keyword("Random String")
    def string(self, length = -1, mode=None):
        """ 如果不传参数默认返回1-50长度之间的随机字符串, mode = [CHARS | SYMBOL | NUM | CN] 分别为字符、符号、数字、中文"""
        length = int(length)
        result = ''
        if length < 0:
            length = random.randint(1, 50)
        data = [['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
                'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'],
                ['!', '@', '#', '$', '%', '^', '&', '*', '(', ')', '[', ']'],
                ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9'],
                ['你', '我', '他', '哈', '黑', '棒', '天', '的', '干', '地', '人', '男', '女', '东', '南', '西', '北', '中', '发', '白', '书', '数', '一', '二', '打', '吹']]
        for _ in range(length):
            if mode == 'CHARS':
                result += random.choice(data[0])
            elif mode == 'SYMBOL':
                result += random.choice(data[1])
            elif mode == 'NUM':
                result += random.choice(data[2])
            elif mode == 'CN':
                result += random.choice(data[3])
            else:
                result += random.choice(random.choice(data))
        return result
    
    @keyword("Random Car Number")
    def license_number(self):
        """ 随机返回一个车牌号 """
        prefix = ['京', '津', '沪', '渝', '冀', '豫',
                  '云', '辽', '黑', '湘', '皖', '鲁',
                  '新', '苏', '浙', '赣', '鄂', '桂',
                  '甘', '晋', '蒙', '陕', '吉', '闽',
                  '贵', '粤', '青', '藏', '川', '宁',
                  '琼']
        result = random.choice(prefix) + random.choice(['A', 'B', 'C'])
        for _ in range(5):
            seed = random.randint(0, 100)
            if seed < 10:
                result += random.choice([chr(i) for i in range(ord('A'),ord('Z')+1)])
            else:
                result += random.choice([str(i) for i in range(0, 10)])
        return result
    
    @keyword("Random Phone")
    def phone(self):
        """ 随机返回一个电话号码 """
        arr = [130,131,132,134,135,136,137,138,139,150,151,152,155,156,157,158,159,182,183,184,185,186,187,188,145,147,170,176,178]
        phone = str(arr[random.randint(0, 28)])
        for _ in range(8):
            phone += str(random.randint(0, 9))
        return phone 
    
    @keyword("Random Email")
    def email(self):
        """ 随机返回一个邮箱地址 """
        names = ['Smith', 'Johnson', 'Williams', 'Brown', 'Jones',
                 'Miller', 'Davis', 'Garcia', 'Rodriguez', 'Wilson', 'Martinez',
                 'Anderson', 'Taylor', 'Thomas', 'Hernandez', 'Moore', 'Martin',
                 'Jackson', 'Thompson', 'White', 'Lopez', 'Lee', 'Gonzalez',
                 'Harris', 'Clark', 'Lewis', 'Robinson', 'Walker',
                 'Perez', 'Hall', 'Young', 'Allen']
        hosts = ['gmail', 'sina', 'qq', '163', 'foxmail',
                 'outlook', 'souche', 'hotmail', 'msn', 'yahoo']
        shuffix = ['.com', '.net', '.org', '.top']
        return random.choice(names) + '@' + random.choice(hosts) + random.choice(shuffix)
    
    @keyword("Random City")
    def city(self):
        """ 随机返回一个城市 """
        cities = ['香港', '澳门','北京市', '天津市', '上海市', '重庆市', '乌鲁木齐','克拉玛依','石河子','阿拉尔市','图木舒克','五家渠','哈密','吐鲁番','阿克苏','喀什','和田','伊宁','塔城','阿勒泰','奎屯','博乐','昌吉','阜康','库尔勒','阿图什','乌苏','拉萨','日喀则','银川','石嘴山','吴忠','固原','中卫','青铜峡市','灵武市','呼和浩特','包头','乌海','赤峰','通辽','鄂尔多斯','呼伦贝尔','巴彦淖尔','乌兰察布','霍林郭勒市','满洲里市','牙克石市','扎兰屯市','根河市','额尔古纳市','丰镇市','锡林浩特市','二连浩特市','乌兰浩特市','阿尔山市','南宁','柳州','桂林','梧州','北海','崇左','来宾','贺州','玉林','百色','河池','钦州','防城港','贵港','岑溪','凭祥','合山','北流','宜州','东兴','桂平','哈尔滨','大庆','齐齐哈尔','佳木斯','鸡西','鹤岗','双鸭山','牡丹江','伊春','七台河','黑河','绥化','五常','双城','尚志','纳河','虎林','密山','铁力','同江','富锦','绥芬河','海林','宁安','穆林','北安','五大连池','肇东','海伦','安达','长春','吉林','四平','辽源','通化','白山','松原','白城','九台市','榆树市','德惠市','舒兰市','桦甸市','蛟河市','磐石市','公主岭市','双辽市','梅河口市','集安市','临江市','大安市','洮南市','延吉市','图们市','敦化市','龙井市','珲春市','和龙市','沈阳','大连','鞍山','抚顺','本溪','丹东','锦州','营口','阜新','辽阳','盘锦','铁岭','朝阳','葫芦岛','新民','瓦房店','普兰','庄河','海城','东港','凤城','凌海','北镇','大石桥','盖州','灯塔','调兵山','开原','凌源','北票','兴城','石家庄','唐山','邯郸','秦皇岛','保定','张家口','承德','廊坊','沧州','衡水','邢台','辛集市','藁城市','晋州市','新乐市','鹿泉市','遵化市','迁安市','武安市','南宫市','沙河市','涿州市','定州市','安国市','高碑店市','泊头市','任丘市','黄骅市','河间市','霸州市','三河市','冀州市','深州市','济南','青岛','淄博','枣庄','东营','烟台','潍坊','济宁','泰安','威海','日照','莱芜','临沂','德州','聊城','菏泽','滨州','章丘','胶南','胶州','平度','莱西','即墨','滕州','龙口','莱阳','莱州','招远','蓬莱','栖霞','海阳','青州','诸城','安丘','高密','昌邑','兖州','曲阜','邹城','乳山','文登','荣成','乐陵','临清','禹城','南京','镇江','常州','无锡','苏州','徐州','连云港','淮安','盐城','扬州','泰州','南通','宿迁','江阴市','宜兴市','邳州市','新沂市','金坛市','溧阳市','常熟市','张家港市','太仓市','昆山市','吴江市','如皋市','通州市','海门市','启东市','东台市','大丰市','高邮市','江都市','仪征市','丹阳市','扬中市','句容市','泰兴市','姜堰市','靖江市','兴化市','合肥','蚌埠','芜湖','淮南','亳州','阜阳','淮北','宿州','滁州','安庆','巢湖','马鞍山','宣城','黄山','池州','铜陵','界首','天长','明光','桐城','宁国','杭州','嘉兴','湖州','宁波','金华','温州','丽水','绍兴','衢州','舟山','台州','建德市','富阳市','临安市','余姚市','慈溪市','奉化市','瑞安市','乐清市','海宁市','平湖市','桐乡市','诸暨市','上虞市','嵊州市','兰溪市','义乌市','东阳市','永康市','江山市','临海市','温岭市','龙泉市','福州','厦门','泉州','三明','南平','漳州','莆田','宁德','龙岩','福清市','长乐市','永安市','石狮市','晋江市','南安市','龙海市','邵武市','武夷山','建瓯市','建阳市','漳平市','福安市','福鼎市','广州','深圳','汕头','惠州','珠海','揭阳','佛山','河源','阳江','茂名','湛江','梅州','肇庆','韶关','潮州','东莞','中山','清远','江门','汕尾','云浮','增城市','从化市','乐昌市','南雄市','台山市','开平市','鹤山市','恩平市','廉江市','雷州市 吴川市','高州市','化州市','高要市','四会市','兴宁市','陆丰市','阳春市','英德市','连州市','普宁市','罗定市','海口','三亚','琼海','文昌','万宁','五指山','儋州','东方','昆明','曲靖','玉溪','保山','昭通','丽江','普洱','临沧','安宁市','宣威市','个旧市','开远市','景洪市','楚雄市','大理市','潞西市','瑞丽市','贵阳','六盘水','遵义','安顺','清镇市','赤水市','仁怀市','铜仁市','毕节市','兴义市','凯里市','都匀市','福泉市','成都','绵阳','德阳','广元','自贡','攀枝花','乐山','南充','内江','遂宁','广安','泸州','达州','眉山','宜宾','雅安','资阳', '都江堰市','彭州市','邛崃市','崇州市','广汉市','什邡市','绵竹市','江油市','峨眉山市','阆中市','华蓥市','万源市','简阳市','西昌市','长沙','株洲','湘潭','衡阳','岳阳','郴州','永州','邵阳','怀化','常德','益阳','张家界','娄底','浏阳市','醴陵市','湘乡市','韶山市','耒阳市','常宁市','武冈市','临湘市','汨罗市','津市市','沅江市','资兴市','洪江市','冷水江市','涟源市','吉首市','武汉','襄樊','宜昌','黄石','鄂州','随州','荆州','荆门','十堰','孝感','黄冈','咸宁','大冶市','丹江口市','洪湖市','石首市','松滋市','宜都市','当阳市','枝江市','老河口市','枣阳市','宜城市','钟祥市','应城市','安陆市','汉川市','麻城市','武穴市','赤壁市','广水市','仙桃市','天门市','潜江市','恩施市','利川市','郑州','洛阳','开封','漯河','安阳','新乡','周口','三门峡','焦作','平顶山','信阳','南阳','鹤壁','濮阳','许昌','商丘','驻马店','巩义市','新郑市','新密市','登封市','荥阳市','偃师市','汝州市','舞钢市','林州市','卫辉市','辉县市','沁阳市','孟州市','禹州市','长葛市','义马市','灵宝市','邓州市','永城市','项城市','济源市','太原','大同','忻州','阳泉','长治','晋城','朔州','晋中','运城','临汾','吕梁','古交','潞城','高平','介休','永济','河津','原平','侯马','霍州','孝义','汾阳','西安','咸阳','铜川','延安','宝鸡','渭南','汉中','安康','商洛','榆林','兴平市','韩城市','华阴市','兰州','天水','平凉','酒泉','嘉峪关','金昌','白银','武威','张掖','庆阳','定西','陇南','玉门市','敦煌市','临夏市','合作市','西宁','格尔木','德令哈','南昌','九江','赣州','吉安','鹰潭','上饶','萍乡','景德镇','新余','宜春','抚州','乐平市','瑞昌市','贵溪市','瑞金市','南康市','井冈山市','丰城市','樟树市','高安市','德兴市','台北','台中','基隆','高雄','台南','新竹','嘉义','板桥市','宜兰市','竹北市','桃园市','苗栗市','丰原市','彰化市','南投市','太保市','斗六市','新营市','凤山市','屏东市','台东市','花莲市','马公市']
        return random.choice(cities)
    
    @keyword("Random Province")
    def province(self):
        """ 随机返回一个省份 """
        provices= ['北京市', '天津市', '上海市', '重庆市', '河北省', '山西省',
                    '辽宁省', '吉林省', '黑龙江省', '江苏省', '浙江省', '安徽省',
                    '福建省', '江西省', '山东省', '河南省', '湖北省', '湖南省',
                    '广东省', '海南省', '四川省', '贵州省', '云南省', '陕西省',
                    '甘肃省', '青海省', '台湾省', '内蒙古自治区', '广西壮族自治区', '西藏自治区',
                    '宁夏回族自治区', '新疆维吾尔自治区', '香港特别行政区', '澳门特别行政区']
        return random.choice(provices)


# -*- coding:utf-8 -*-
import os
import imp

def hook_zh():
    robot_path = imp.find_module("robot")[1]
    print(robot_path)
    
    if os.path.exists(robot_path + '/HOOK_ZH'):
        return

    try:
        os.rename(robot_path + "/utils/unic.py", robot_path + "/utils/unic_back.py")
        fo = open(robot_path + "/utils/unic.py", "w")
        fo.write(code)
        print('Hook unic.py成功！')
        fo.close()
    except:
        print('Hook unic.py失败！')
        return

    open(robot_path + '/HOOK_ZH', 'a').close()
    

code ="""#  Copyright 2008-2015 Nokia Networks
#  Copyright 2016-     Robot Framework Foundation
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from pprint import PrettyPrinter

from .platform import IRONPYTHON, JYTHON, PY2
from .robottypes import is_bytes, is_unicode
import json

if PY2:

    def unic(item):
        if isinstance(item, unicode):
            return item
        if isinstance(item, (bytes, bytearray)):
            try:
                return item.decode('ASCII')
            except UnicodeError:
                return u''.join(chr(b) if b < 128 else '\\\\x%x' % b
                                for b in bytearray(item))
        if isinstance(item, (list, dict, tuple)):
            try:
                item = json.dumps(item, ensure_ascii=False, encoding='utf-8')
            except UnicodeDecodeError:
                try:
                    item = json.dumps(item, ensure_ascii=False, encoding='utf-8')
                except:
                    pass
            except:
                pass
        try:
            try:
                return unicode(item)
            except UnicodeError:
                return unic(str(item))
        except:
            return _unrepresentable_object(item)

else:

    def unic(item):
        if isinstance(item, str):
            return item
        if isinstance(item, (bytes, bytearray)):
            try:
                return item.decode('ASCII')
            except UnicodeError:
                return ''.join(chr(b) if b < 128 else '\\\\x%x' % b
                               for b in item)
        try:
            return str(item)
        except:
            return _unrepresentable_object(item)


# JVM and .NET seem to handle Unicode normalization automatically. Importing
# unicodedata on Jython also takes some time so it's better to avoid it.
if not (JYTHON or IRONPYTHON):

    from unicodedata import normalize
    _unic = unic

    def unic(item):
        return normalize('NFC', _unic(item))


def prepr(item, width=400):
    return unic(PrettyRepr(width=width).pformat(item))


class PrettyRepr(PrettyPrinter):

    def format(self, object, context, maxlevels, level):
        try:
            if is_unicode(object):
                return repr(object).lstrip('u'), True, False
            if is_bytes(object):
                return 'b' + repr(object).lstrip('b'), True, False
            return PrettyPrinter.format(self, object, context, maxlevels, level)
        except:
            return _unrepresentable_object(object), True, False


def _unrepresentable_object(item):
    from .error import get_error_message
    return u"<Unrepresentable object %s. Error: %s>" \\
           % (item.__class__.__name__, get_error_message())
"""


from .db import DBKeywords
from .loglistener import LogListener
from .requester import RequesterKeywords
from .validator import ValidatorKeywords
from .randomkeywords import RandomKeywords
from .select import SelectKeywords



# -*- coding:utf-8 -*-

from SCLibrary.zk import ZookeeperRegistry
from robot.libraries.BuiltIn import BuiltIn
from SCLibrary.fork_requests import RequestsKeywords
from SCLibrary.base import keyword
from json import dumps
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

class RequesterKeywords(RequestsKeywords):

    def __init__(self):
        super(RequesterKeywords, self).__init__()
        self.response = None
        self.zk = None

    @keyword("HTTP Create Session")
    def http_create_session(self, alias, url, headers={}, cookies={},
                            auth=None, timeout=None, proxies=None,
                            verify=False, debug=0, max_retries=3, backoff_factor=0.10, disable_warnings=0):
        """ 创建一个Http的Session：

        ``url`` 域名

        ``alias`` Session的名字

        ``headers`` Headers

        ``cookies`` Cookies

        ``auth`` List of username & password for HTTP Basic Auth

        ``timeout`` Connection timeout

        ``proxies`` Dictionary that contains proxy urls for HTTP and HTTPS communication

        ``max_retries`` 最大重试次数，默认3次

        ``backoff_factor`` 重试间隔时间，默认0.1秒
        """
        if not 'Content-Type' in headers:
            headers['Content-Type'] = 'application/json'
        if not 'charset' in headers:
            headers['charset'] = 'utf-8'
        if not 'Accept-Encoding' in headers:
            headers['Accept-Encoding'] = 'gzip'
        if not 'Accept-Language' in headers:
            headers['Accept-Language'] = 'zh-CN'
        try:
            sc_token = BuiltIn().get_variable_value("${SC_TOKEN}")
            if not 'TT' in headers:
                headers['TT'] = sc_token
            if not 'souche-inc-security-token' in headers:
                headers['souche-inc-security-token'] = sc_token
            if not '_security_token_inc' in headers:
                headers['_security_token_inc'] = sc_token
            if not 'token' in headers:
                headers['token'] = sc_token
            if not '_security_token' in headers:
                headers['_security_token'] = sc_token
            if not 'souche-security-token' in headers:
                headers['souche-security-token'] = sc_token
        except:
            print('警告！ 没有找到全局变量：SC_TOKEN')
            pass
        return super(RequesterKeywords, self).create_session(alias,
                                                             url,
                                                             headers,
                                                             cookies,
                                                             auth,
                                                             timeout,
                                                             proxies,
                                                             verify,
                                                             debug,
                                                             max_retries,
                                                             backoff_factor,
                                                             disable_warnings)

    @keyword("HTTP Post")
    def http_post(self,
                  alias,
                  uri,
                  params=None,
                  headers=None,
                  data=None,
                  json=None,
                  files=None,
                  allow_redirects=None,
                  timeout=None):
        """ 通过Session Alias发起一个Post请求

        ``alias`` 指定Session

        ``uri`` 类似 /api/v1/xxx.json 的部分

        ``params`` 参数，拼接在URL上

        ``data``  参数

        ``headers`` Headers

        ``timeout`` connection timeout
        """
        return self.http_post_request(alias,
                                      uri,
                                      params,
                                      headers,
                                      data,
                                      json,
                                      files,
                                      allow_redirects,
                                      timeout)

    # @keyword("HTTP Post Request")
    def http_post_request(self,
                          alias,
                          uri,
                          params=None,
                          headers=None,
                          data=None,
                          json=None,
                          files=None,
                          allow_redirects=None,
                          timeout=None):
        self.response = None
        self.response = super(RequesterKeywords, self).post_request(alias,
                                                                    uri,
                                                                    data,
                                                                    json,
                                                                    params,
                                                                    headers,
                                                                    files,
                                                                    allow_redirects,
                                                                    timeout)
        return self.response.json()

    @keyword("HTTP Delete")
    def http_delete_request(self,
                            alias,
                            uri,
                            params=None,
                            headers=None,
                            data=None,
                            json=None,
                            allow_redirects=None,
                            timeout=None):
        """ 通过Session Alias发起一个Delete请求

        入参参考 HTTP Post
        """
        self.response = None
        self.response = super(RequesterKeywords, self).delete_request(alias=alias,
                                                                      uri=uri,
                                                                      data=data,
                                                                      json=json,
                                                                      params=params,
                                                                      headers=headers,
                                                                      allow_redirects=allow_redirects,
                                                                      timeout=timeout)
        return self.response.json()

    @keyword("HTTP Put")
    def http_put_request(self,
                         alias,
                         uri,
                         params=None,
                         headers=None,
                         data=None,
                         json=None,
                         files=None,
                         allow_redirects=None,
                         timeout=None):
        """ 通过Session Alias发起一个Put请求

        入参参考 HTTP Post
        """
        self.response = None
        self.response = super(RequesterKeywords, self).put_request(alias=alias,
                                                                   uri=uri,
                                                                   data=data,
                                                                   json=json,
                                                                   params=params,
                                                                   headers=headers,
                                                                   files=files,
                                                                   allow_redirects=allow_redirects,
                                                                   timeout=timeout)
        return self.response.json()

    @keyword("HTTP Get")
    def http_get(self,
                 alias,
                 uri,
                 params=None,
                 headers=None,
                 json=None,
                 allow_redirects=None,
                 timeout=None):
        """ 通过Session Alias发起一个Get请求

        ``alias`` 指定Session

        ``uri`` 类似 /api/v1/xxx.json 的部分

        ``params`` 参数

        ``headers`` Headers

        ``timeout`` connection timeout
        """
        return self.http_get_request(alias,
                                     uri,
                                     params,
                                     headers,
                                     json,
                                     allow_redirects,
                                     timeout)

    # @keyword("HTTP Get Request")
    def http_get_request(self,
                         alias,
                         uri,
                         params=None,
                         headers=None,
                         json=None,
                         allow_redirects=None,
                         timeout=None):
        self.response = None
        self.response = super(RequesterKeywords, self).get_request(alias,
                                                                   uri,
                                                                   headers,
                                                                   json,
                                                                   params,
                                                                   allow_redirects,
                                                                   timeout)
        return self.response.json()

    @keyword("HTTP Delete All Sessions")
    def http_delete_all_session(self):
        """ 清除所有的Session """
        super(RequesterKeywords, self).delete_all_sessions()

    @keyword("HTTP Update Session")
    def http_update_session(self, alias, headers=None, cookies=None):
        """ 更新Session 
        ``alias`` Session名字

        ``headers`` Headers

        ``cookies`` Cookies
        """
        super(RequesterKeywords, self).update_session(alias, headers, cookies)

    @keyword("HTTP Code Should Be")
    def http_code_should_be(self, code):
        """ 断言Code的值，这个code指{success: , code: , data: } 这个返回体的code
        ``code`` 大搜车状态码
        """
        resp_code = self.get_response_value(
            'code', '该关键词 仅适用于类似 {code: xxx} 返回结构')
        if(resp_code != code):
            raise AssertionError("%s != %s" % (code, resp_code))

    @keyword("HTTP Check if Success")
    def http_check_if_success(self):
        """ 断言请求是否成功，依据{success: , code: , data: } 这个返回体的success值
        """
        success = self.get_response_value(
            'success', '该关键词 仅适用于类似 {success: boolean} 返回结构')
        if(success == False):
            raise AssertionError("期望请求成功，但是现在是失败!")

    @keyword("HTTP Check if Failure")
    def http_check_if_failure(self):
        """ 断言请求是否失败，依据{success: , code: , data: } 这个返回体的success值
        """
        success = self.get_response_value(
            'success', '该关键词 仅适用于类似 {success: boolean} 返回结构')
        if(success == True):
            raise AssertionError("期望请求失败，但是现在是成功!")

    @keyword("DUBBO Request")
    def dubbo_post(self, interface, method, token='souche_http_token', type='form', params=None, headers=None):
        """ Dubbo请求
        ``interface`` 微服务接口名

        ``method``   方法名

        ``token``    token，默认souche_http_token

        ``type``     请求头的Content-Type，可选值：form, json， 默认form (application/x-www-form-urlencoded)

        ``params``   参数

        ``headers``  额外需要添加的Headers
        """
        if self.zk is None:
            built_in = BuiltIn()
            rf_zk = built_in.get_variable_value("${RF_ZK}")
            self.zk = ZookeeperRegistry(rf_zk)

        url = self.zk.subscribe(interface)[0]

        session_headers = {}
        if type == 'form':
            # type=json时， Content-Type默认为application/json
            session_headers['Content-Type'] = 'application/x-www-form-urlencoded'
        self.http_create_session(
            alias='dubbo', url='http://%s' % url, headers=session_headers)
        headers = {'_method_name': method, '_dubbo_token': token}
        return self.http_post(alias='dubbo', uri='/%s' % interface, data=params, headers=headers)

    @keyword("DB Mongo Query")
    def db_mongo_query(self, host, name, query, headers=None, page_size=1):
        """ 从Mongo开放的Http api上搜索内容， 不需要DB Connect
        ``host`` Mongo服务的Host

        ``name`` 表名

        ``query`` 查询语句，类似 {_id:"${car_id}"}
        
        ``page_size`` 分页， 默认1
        """
        if headers is None:
            headers = {}
        if not 'X-Requested-With' in headers:
            headers['X-Requested-With'] = 'XMLHttpRequest'
        self.http_create_session(alias='mongo', url=host, headers=headers)
        params = {'query': query, 'collectionName': name,
                  'pageSize': page_size}
        return self.http_post(alias='mongo', uri='/manage/dataManage/findPage.json', params=params, headers=headers)

    @keyword("String To Json")
    def http_to_json(self, content, pretty_print=False):
        """ 字符串转成Json对象， Json对象可以使用${json['xx'][0]}语法
        ``content`` 字符串内容
        """
        return super(RequesterKeywords, self).to_json(alias, headers, cookies)

    @keyword("To String")
    def http_to_string(self, list_or_dict):
        """ 字典、数组转成Json字符串
        ``list_or_dict`` 数组或字典
        """
        return dumps(list_or_dict, ensure_ascii=False)

    def get_response_value(self, key, exception):
        json = self.response.json()
        if(not key in json):
            raise AssertionError(exception)
        return json[key]



# -*- coding:utf-8 -*-
# 这里fork了  0.5.0的RequestsLibrary

import json
import types
import sys

import requests
from requests.sessions import merge_setting
from requests.cookies import merge_cookies
import logging
from requests.packages.urllib3.util import Retry
import robot
from robot.api import logger
from robot.libraries.BuiltIn import BuiltIn

from .compat import httplib, urlencode, PY3
try:
    from requests_ntlm import HttpNtlmAuth
except ImportError:
    pass


class WritableObject:
    ''' HTTP stream handler '''

    def __init__(self):
        self.content = []

    def write(self, string):
        self.content.append(string)


class RequestsKeywords(object):
    """``RequestsLibrary`` is a [http://code.google.com/p/robotframework/|Robot Framework] test library that uses the [https://github.com/kennethreitz/requests|Requests] HTTP client.

    Here is an example testcase

    | ***** Settings *****   |                                 |                     |                       |               |
    | Library                | Collections                     |                     |                       |               |
    | Library                | RequestsLibrary                 |                     |                       |               |
    | ***** Test Cases ***** |                                 |                     |                       |               |
    | Get Requests           |                                 |                     |                       |               |
    |                        | Create Session                  | github              | http://api.github.com |               |
    |                        | Create Session                  | google              | http://www.google.com |               |
    |                        | ${resp}=                        | Get Request         | google                | /             |
    |                        | Should Be Equal As Strings      | ${resp.status_code} | 200                   |               |
    |                        | ${resp}=                        | Get Request         | github                | /users/bulkan |
    |                        | Should Be Equal As Strings      | ${resp.status_code} | 200                   |               |
    |                        | Dictionary Should Contain Value | ${resp.json()}      | Bulkan Savun Evcimen  |               |
    """
    ROBOT_LIBRARY_SCOPE = 'Global'

    def __init__(self):
        self._cache = robot.utils.ConnectionCache('No sessions created')
        self.builtin = BuiltIn()
        self.debug = 0

    def console(self, message):
        # logger.console('\033[33m%s\033[0m' % message)
        logger.console('%s' % message)

    def _create_session(
            self,
            alias,
            url,
            headers,
            cookies,
            auth,
            timeout,
            max_retries,
            backoff_factor,
            proxies,
            verify,
            debug,
            disable_warnings):
        """ Create Session: create a HTTP session to a server

        ``url`` Base url of the server

        ``alias`` Robot Framework alias to identify the session

        ``headers`` Dictionary of default headers

        ``auth`` List of username & password for HTTP Basic Auth

        ``timeout`` Connection timeout

        ``max_retries`` The maximum number of retries each connection should attempt.

        ``backoff_factor`` The pause between for each retry

        ``proxies`` Dictionary that contains proxy urls for HTTP and HTTPS communication

        ``verify`` Whether the SSL cert will be verified. A CA_BUNDLE path can also be provided.

        ``debug`` Enable http verbosity option more information
                https://docs.python.org/2/library/httplib.html#httplib.HTTPConnection.set_debuglevel

        ``disable_warnings`` Disable requests warning useful when you have large number of testcases
        """

        self.builtin.log('Creating session: %s' % alias, 'DEBUG')
        s = session = requests.Session()
        s.headers.update(headers)
        s.auth = auth if auth else s.auth
        s.proxies = proxies if proxies else s.proxies

        try:
            max_retries = int(max_retries)
        except ValueError as err:
            raise ValueError("Error converting max_retries parameter: %s"   % err)

        if max_retries > 0:
            http = requests.adapters.HTTPAdapter(max_retries=Retry(total=max_retries, backoff_factor=backoff_factor))
            https = requests.adapters.HTTPAdapter(max_retries=Retry(total=max_retries, backoff_factor=backoff_factor))

            # Replace the session's original adapters
            s.mount('http://', http)
            s.mount('https://', https)

        # Disable requests warnings, useful when you have large number of testcase
        # you will observe drastical changes in Robot log.html and output.xml files size
        if disable_warnings:
            logging.basicConfig() # you need to initialize logging, otherwise you will not see anything from requests
            logging.getLogger().setLevel(logging.ERROR)
            requests_log = logging.getLogger("requests")
            requests_log.setLevel(logging.ERROR)
            requests_log.propagate = True
            if not verify:
                requests.packages.urllib3.disable_warnings()

        # verify can be a Boolean or a String
        if isinstance(verify, bool):
            s.verify = verify
        elif isinstance(verify, str) or isinstance(verify, unicode):
            if verify.lower() == 'true' or verify.lower() == 'false':
                s.verify = self.builtin.convert_to_boolean(verify)
            else:
                # String for CA_BUNDLE, not a Boolean String
                s.verify = verify
        else:
            # not a Boolean nor a String
            s.verify = verify

        # cant pass these into the Session anymore
        self.timeout = float(timeout) if timeout is not None else None
        self.cookies = cookies
        self.verify = verify if self.builtin.convert_to_boolean(verify) != True else None

        s.url = url

        # Enable http verbosity
        if int(debug) >= 1:
            self.debug = int(debug)
            httplib.HTTPConnection.debuglevel = self.debug

        self._cache.register(session, alias=alias)
        return session

    def create_session(self, alias, url, headers={}, cookies=None,
                       auth=None, timeout=None, proxies=None,
                       verify=False, debug=0, max_retries=3, backoff_factor=0.10, disable_warnings=0):
        """ Create Session: create a HTTP session to a server

        ``url`` Base url of the server

        ``alias`` Robot Framework alias to identify the session

        ``headers`` Dictionary of default headers

        ``auth`` List of username & password for HTTP Basic Auth

        ``timeout`` Connection timeout

        ``proxies`` Dictionary that contains proxy urls for HTTP and HTTPS communication

        ``verify`` Whether the SSL cert will be verified. A CA_BUNDLE path can also be provided.
                 Defaults to False.

        ``debug`` Enable http verbosity option more information
                https://docs.python.org/2/library/httplib.html#httplib.HTTPConnection.set_debuglevel

        ``max_retries`` The maximum number of retries each connection should attempt.

        ``backoff_factor`` The pause between for each retry

        ``disable_warnings`` Disable requests warning useful when you have large number of testcases
        """
        auth = requests.auth.HTTPBasicAuth(*auth) if auth else None

        try:
            self._cache.get_connection(alias)
        except:
            self.console('\n［创建Session］ : %s=%s, headers=%s' % ( alias, url, headers))

        return self._create_session(
            alias,
            url,
            headers,
            cookies,
            auth,
            timeout,
            max_retries,
            backoff_factor,
            proxies,
            verify,
            debug,
            disable_warnings)

    def create_custom_session(
            self,
            alias,
            url,
            auth,
            headers={},
            cookies=None,
            timeout=None,
            proxies=None,
            verify=False,
            debug=0,
            max_retries=3,
            backoff_factor=0.10,
            disable_warnings=0):
        """ Create Session: create a HTTP session to a server

        ``url`` Base url of the server

        ``alias`` Robot Framework alias to identify the session

        ``headers`` Dictionary of default headers

        ``auth`` A Custom Authentication object to be passed on to the reqests library.
                http://docs.python-requests.org/en/master/user/advanced/#custom-authentication

        ``timeout`` Connection timeout

        ``proxies`` Dictionary that contains proxy urls for HTTP and HTTPS communication

        ``verify`` Whether the SSL cert will be verified. A CA_BUNDLE path can also be provided.
                 Defaults to False.

        ``debug`` Enable http verbosity option more information
                https://docs.python.org/2/library/httplib.html#httplib.HTTPConnection.set_debuglevel

        ``max_retries`` The maximum number of retries each connection should attempt.

        ``backoff_factor`` The pause between for each retry

        ``disable_warnings`` Disable requests warning useful when you have large number of testcases
        """

        logger.info('Creating Custom Authenticated Session using : alias=%s, url=%s, headers=%s, \
                    cookies=%s, auth=%s, timeout=%s, proxies=%s, verify=%s, \
                    debug=%s ' % (alias, url, headers, cookies, auth, timeout,
                                  proxies, verify, debug))

        return self._create_session(
            alias,
            url,
            headers,
            cookies,
            auth,
            timeout,
            max_retries,
            backoff_factor,
            proxies,
            verify,
            debug,
            disable_warnings)

    def create_ntlm_session(
            self,
            alias,
            url,
            auth,
            headers={},
            cookies=None,
            timeout=None,
            proxies=None,
            verify=False,
            debug=0,
            max_retries=3,
            backoff_factor=0.10,
            disable_warnings=0):
        """ Create Session: create a HTTP session to a server

        ``url`` Base url of the server

        ``alias`` Robot Framework alias to identify the session

        ``headers`` Dictionary of default headers

        ``auth`` ['DOMAIN', 'username', 'password'] for NTLM Authentication

        ``timeout`` Connection timeout

        ``proxies`` Dictionary that contains proxy urls for HTTP and HTTPS communication

        ``verify`` Whether the SSL cert will be verified. A CA_BUNDLE path can also be provided.
                 Defaults to False.

        ``debug`` Enable http verbosity option more information
                https://docs.python.org/2/library/httplib.html#httplib.HTTPConnection.set_debuglevel

        ``max_retries`` The maximum number of retries each connection should attempt.

        ``backoff_factor`` The pause between for each retry

        ``disable_warnings`` Disable requests warning useful when you have large number of testcases
        """
        if not HttpNtlmAuth:
            raise AssertionError('Requests NTLM module not loaded')
        elif len(auth) != 3:
            raise AssertionError('Incorrect number of authentication arguments'
                                 ' - expected 3, got {}'.format(len(auth)))
        else:
            ntlm_auth = HttpNtlmAuth('{}\\{}'.format(auth[0], auth[1]),
                                     auth[2])
            logger.info('Creating NTLM Session using : alias=%s, url=%s, \
                        headers=%s, cookies=%s, ntlm_auth=%s, timeout=%s, \
                        proxies=%s, verify=%s, debug=%s '
                        % (alias, url, headers, cookies, ntlm_auth,
                           timeout, proxies, verify, debug))

            return self._create_session(
                alias,
                url,
                headers,
                cookies,
                ntlm_auth,
                timeout,
                max_retries,
                backoff_factor,
                proxies,
                verify,
                debug,
                disable_warnings)

    def create_digest_session(self, alias, url, auth, headers={}, cookies=None,
                              timeout=None, proxies=None, verify=False,
                              debug=0, max_retries=3,backoff_factor=0.10, disable_warnings=0):
        """ Create Session: create a HTTP session to a server

        ``url`` Base url of the server

        ``alias`` Robot Framework alias to identify the session

        ``headers`` Dictionary of default headers

        ``auth`` ['DOMAIN', 'username', 'password'] for NTLM Authentication

        ``timeout`` Connection timeout

        ``proxies`` Dictionary that contains proxy urls for HTTP and HTTPS communication

        ``verify`` Whether the SSL cert will be verified. A CA_BUNDLE path can also be provided.
                 Defaults to False.

        ``debug`` Enable http verbosity option more information
                https://docs.python.org/2/library/httplib.html#httplib.HTTPConnection.set_debuglevel

        ``max_retries`` The maximum number of retries each connection should attempt.

        ``backoff_factor`` The pause between for each retry

        ``disable_warnings`` Disable requests warning useful when you have large number of testcases
        """
        digest_auth = requests.auth.HTTPDigestAuth(*auth) if auth else None

        return self._create_session(
            alias,
            url,
            headers,
            cookies,
            digest_auth,
            timeout,
            max_retries,
            backoff_factor,
            proxies,
            verify,
            debug,
            disable_warnings)

    def create_client_cert_session(self, alias, url, headers={}, cookies=None,
                       client_certs=None, timeout=None, proxies=None,
                       verify=False, debug=0, max_retries=3, backoff_factor=0.10, disable_warnings=0):
        """ Create Session: create a HTTP session to a server

        ``url`` Base url of the server

        ``alias`` Robot Framework alias to identify the session

        ``headers`` Dictionary of default headers

        ``client_certs`` ['client certificate', 'client key'] PEM files containing the client key and certificate

        ``timeout`` Connection timeout

        ``proxies`` Dictionary that contains proxy urls for HTTP and HTTPS communication

        ``verify`` Whether the SSL cert will be verified. A CA_BUNDLE path can also be provided.
                 Defaults to False.

        ``debug`` Enable http verbosity option more information
                https://docs.python.org/2/library/httplib.html#httplib.HTTPConnection.set_debuglevel

        ``max_retries`` The maximum number of retries each connection should attempt.

        ``backoff_factor`` The pause between for each retry

        ``disable_warnings`` Disable requests warning useful when you have large number of testcases
        """

        logger.info('Creating Session using : alias=%s, url=%s, headers=%s, \
                    cookies=%s, client_certs=%s, timeout=%s, proxies=%s, verify=%s, \
                    debug=%s ' % (alias, url, headers, cookies, client_certs, timeout,
                                  proxies, verify, debug))

        session = self._create_session(
            alias,
            url,
            headers,
            cookies,
            None,
            timeout,
            max_retries,
            backoff_factor,
            proxies,
            verify,
            debug,
            disable_warnings)

        session.cert = tuple(client_certs)
        return session

    def delete_all_sessions(self):
        """ Removes all the session objects """
        logger.info('Delete All Sessions')

        self._cache.empty_cache()

    def update_session(self, alias, headers=None, cookies=None):
        """Update Session Headers: update a HTTP Session Headers

        ``alias`` Robot Framework alias to identify the session

        ``headers`` Dictionary of headers merge into session
        """
        session = self._cache.switch(alias)
        session.headers = merge_setting(headers, session.headers)
        session.cookies = merge_cookies(session.cookies, cookies)

    def to_json(self, content, pretty_print=False):
        """ Convert a string to a JSON object

        ``content`` String content to convert into JSON

        ``pretty_print`` If defined, will output JSON is pretty print format
        """
        if PY3:
            if isinstance(content, bytes):
                content = content.decode(encoding='utf-8')
        if pretty_print:
            json_ = self._json_pretty_print(content)
        else:
            json_ = json.loads(content)
        logger.info('To JSON using : content=%s ' % (content))
        logger.info('To JSON using : pretty_print=%s ' % (pretty_print))

        return json_

    def get_request(
            self,
            alias,
            uri,
            headers=None,
            json=None,
            params=None,
            allow_redirects=None,
            timeout=None):
        """ Send a GET request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the GET request to

        ``params`` url parameters to append to the uri

        ``headers`` a dictionary of headers to use with the request

        ``json`` json data to send in the body of the :class:`Request`.

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``timeout`` connection timeout
        """
        session = self._cache.switch(alias)

        self.console('\n>> ［请求地址］:（%s） %s' % (alias, uri))
        if headers is not None and len(headers) > 0:
            self.console('\n>> ［额外Headers］:%s' % headers)
        if params is not None and len(params) > 0:
            self.console('\n>> ［入参］:%s' % params)

        redir = True if allow_redirects is None else allow_redirects

        response = self._get_request(
            session, uri, params, headers, json, redir, timeout)

        self.console('\n<< ［返回］:%s' % response.content.decode('utf-8'))
        # logger.info(
        #     'Get Request using : alias=%s, uri=%s, headers=%s json=%s' %
        #     (alias, uri, headers, json))

        return response

    def get(
            self,
            alias,
            uri,
            params=None,
            headers=None,
            allow_redirects=None,
            timeout=None):
        """ **Deprecated- See Get Request now**

        Send a GET request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the GET request to

        ``headers`` a dictionary of headers to use with the request

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``timeout`` connection timeout
        """
        logger.warn("Deprecation Warning: Use Get Request in the future")
        session = self._cache.switch(alias)

        redir = True if allow_redirects is None else allow_redirects

        response = self._get_request(
            session, uri, params, headers, redir, timeout, json)

        return response

    def post_request(
            self,
            alias,
            uri,
            data=None,
            json=None,
            params=None,
            headers=None,
            files=None,
            allow_redirects=None,
            timeout=None):
        """ Send a POST request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the POST request to

        ``data`` a dictionary of key-value pairs that will be urlencoded
               and sent as POST data
               or binary data that is sent as the raw body content
               or passed as such for multipart form data if ``files`` is also
                  defined

        ``json`` a value that will be json encoded
               and sent as POST data if files or data is not specified

        ``params`` url parameters to append to the uri

        ``headers`` a dictionary of headers to use with the request

        ``files`` a dictionary of file names containing file data to POST to the server

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``timeout`` connection timeout
        """            
        session = self._cache.switch(alias)

        self.console('\n>> ［请求地址］:（%s） %s' % (alias, uri))
        if headers is not None and len(headers) > 0:
            self.console('\n>> ［额外Headers］:%s' % headers)
        if params is not None and len(params) > 0:
            self.console('\n>> ［入参］:%s' % params)
        if data is not None and len(data) > 0:
            self.console('\n>> ［入参］:%s' % data)
        

        if not files:
            data = self._format_data_according_to_header(session, data, headers)
        redir = True if allow_redirects is None else allow_redirects

        response = self._body_request(
            "post",
            session,
            uri,
            data,
            json,
            params,
            files,
            headers,
            redir,
            timeout)
        dataStr = self._format_data_to_log_string_according_to_header(data, headers)
        # logger.info('Post Request using : alias=%s, uri=%s, data=%s, headers=%s, files=%s, allow_redirects=%s '
        #             % (alias, uri, dataStr, headers, files, redir))
      
        self.console('\n<< ［返回］:%s' % response.content.decode('utf-8'))
        return response

    def post(
            self,
            alias,
            uri,
            data={},
            headers=None,
            files=None,
            allow_redirects=None,
            timeout=None):
        """ **Deprecated- See Post Request now**

        Send a POST request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the GET request to

        ``data`` a dictionary of key-value pairs that will be urlencoded
               and sent as POST data
               or binary data that is sent as the raw body content

        ``headers`` a dictionary of headers to use with the request

        ``files`` a dictionary of file names containing file data to POST to the server

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``timeout`` connection timeout
        """
        logger.warn("Deprecation Warning: Use Post Request in the future")
        session = self._cache.switch(alias)
        data = self._utf8_urlencode(data)
        redir = True if allow_redirects is None else allow_redirects

        response = self._body_request(
            "post",
            session,
            uri,
            data,
            None,
            None,
            files,
            headers,
            redir,
            timeout)

        return response

    def patch_request(
            self,
            alias,
            uri,
            data=None,
            json=None,
            params=None,
            headers=None,
            files=None,
            allow_redirects=None,
            timeout=None):
        """ Send a PATCH request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the PATCH request to

        ``data`` a dictionary of key-value pairs that will be urlencoded
               and sent as PATCH data
               or binary data that is sent as the raw body content

        ``json`` a value that will be json encoded
               and sent as PATCH data if data is not specified

        ``headers`` a dictionary of headers to use with the request

        ``files`` a dictionary of file names containing file data to PATCH to the server

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``params`` url parameters to append to the uri

        ``timeout`` connection timeout
        """
        session = self._cache.switch(alias)
        data = self._format_data_according_to_header(session, data, headers)
        redir = True if allow_redirects is None else allow_redirects

        response = self._body_request(
            "patch",
            session,
            uri,
            data,
            json,
            params,
            files,
            headers,
            redir,
            timeout)

        if isinstance(data, bytes):
            data = data.decode('utf-8')
        logger.info('Patch Request using : alias=%s, uri=%s, data=%s, \
                    headers=%s, files=%s, allow_redirects=%s '
                    % (alias, uri, data, headers, files, redir))

        return response

    def patch(
            self,
            alias,
            uri,
            data={},
            headers=None,
            files={},
            allow_redirects=None,
            timeout=None):
        """ **Deprecated- See Patch Request now**

        Send a PATCH request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the PATCH request to

        ``data`` a dictionary of key-value pairs that will be urlencoded
               and sent as PATCH data
               or binary data that is sent as the raw body content

        ``headers`` a dictionary of headers to use with the request

        ``files`` a dictionary of file names containing file data to PATCH to the server

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``timeout`` connection timeout
        """
        logger.warn("Deprecation Warning: Use Patch Request in the future")
        session = self._cache.switch(alias)
        data = self._utf8_urlencode(data)
        redir = True if allow_redirects is None else allow_redirects

        response = self._body_request(
            "patch",
            session,
            uri,
            data,
            None,
            None,
            files,
            headers,
            redir,
            timeout)

        return response

    def put_request(
            self,
            alias,
            uri,
            data=None,
            json=None,
            params=None,
            files=None,
            headers=None,
            allow_redirects=None,
            timeout=None):
        """ Send a PUT request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the PUT request to

        ``data`` a dictionary of key-value pairs that will be urlencoded
               and sent as PUT data
               or binary data that is sent as the raw body content

        ``json`` a value that will be json encoded
               and sent as PUT data if data is not specified

        ``headers`` a dictionary of headers to use with the request

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``params`` url parameters to append to the uri

        ``timeout`` connection timeout
        """
        session = self._cache.switch(alias)

        self.console('\n>> ［请求地址］:（%s） %s' % (alias, uri))
        if headers is not None and len(headers) > 0:
            self.console('\n>> ［额外Headers］:%s' % headers)
        if params is not None and len(params) > 0:
            self.console('\n>> ［入参］:%s' % params)
        if data is not None and len(data) > 0:
            self.console('\n>> ［入参］:%s' % data)

        data = self._format_data_according_to_header(session, data, headers)
        redir = True if allow_redirects is None else allow_redirects

        response = self._body_request(
            "put",
            session,
            uri,
            data,
            json,
            params,
            files,
            headers,
            redir,
            timeout)

        self.console('\n<< ［返回］:%s' % response.content.decode('utf-8'))
        return response

    def put(
            self,
            alias,
            uri,
            data=None,
            headers=None,
            allow_redirects=None,
            timeout=None):
        """ **Deprecated- See Put Request now**

        Send a PUT request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the PUT request to

        ``headers`` a dictionary of headers to use with the request

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``timeout`` connection timeout
        """
        logger.warn("Deprecation Warning: Use Put Request in the future")
        session = self._cache.switch(alias)
        data = self._utf8_urlencode(data)
        redir = True if allow_redirects is None else allow_redirects

        response = self._body_request(
            "put",
            session,
            uri,
            data,
            None,
            None,
            None,
            headers,
            redir,
            timeout)

        return response

    def delete_request(
            self,
            alias,
            uri,
            data=None,
            json=None,
            params=None,
            headers=None,
            allow_redirects=None,
            timeout=None):
        """ Send a DELETE request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the DELETE request to

        ``json`` a value that will be json encoded
               and sent as request data if data is not specified

        ``headers`` a dictionary of headers to use with the request

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``timeout`` connection timeout
        """
        session = self._cache.switch(alias)

        self.console('\n>> ［请求地址］:（%s） %s' % (alias, uri))
        if headers is not None and len(headers) > 0:
            self.console('\n>> ［额外Headers］:%s' % headers)
        if params is not None and len(params) > 0:
            self.console('\n>> ［入参］:%s' % params)
        if data is not None and len(data) > 0:
            self.console('\n>> ［入参］:%s' % data)

        data = self._format_data_according_to_header(session, data, headers)
        redir = True if allow_redirects is None else allow_redirects

        response = self._delete_request(
            session, uri, data, json, params, headers, redir, timeout)

        self.console('\n<< ［返回］:%s' % response.content.decode('utf-8'))
        return response

    def delete(
            self,
            alias,
            uri,
            data=(),
            headers=None,
            allow_redirects=None,
            timeout=None):
        """ * * *   Deprecated- See Delete Request now   * * *

        Send a DELETE request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the DELETE request to

        ``headers`` a dictionary of headers to use with the request

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``timeout`` connection timeout
        """
        logger.warn("Deprecation Warning: Use Delete Request in the future")
        session = self._cache.switch(alias)
        data = self._utf8_urlencode(data)
        redir = True if allow_redirects is None else allow_redirects

        response = self._delete_request(
            session, uri, data, json, None, headers, redir, timeout)

        return response

    def head_request(
            self,
            alias,
            uri,
            headers=None,
            allow_redirects=None,
            timeout=None):
        """ Send a HEAD request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the HEAD request to

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``headers`` a dictionary of headers to use with the request
        """
        session = self._cache.switch(alias)
        redir = False if allow_redirects is None else allow_redirects
        response = self._head_request(session, uri, headers, redir, timeout)
        logger.info('Head Request using : alias=%s, uri=%s, headers=%s, \
        allow_redirects=%s ' % (alias, uri, headers, redir))

        return response

    def head(
            self,
            alias,
            uri,
            headers=None,
            allow_redirects=None,
            timeout=None):
        """ **Deprecated- See Head Request now**

        Send a HEAD request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the HEAD request to

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``headers`` a dictionary of headers to use with the request
        """
        logger.warn("Deprecation Warning: Use Head Request in the future")
        session = self._cache.switch(alias)
        redir = False if allow_redirects is None else allow_redirects
        response = self._head_request(session, uri, headers, redir, timeout)

        return response

    def options_request(
            self,
            alias,
            uri,
            headers=None,
            allow_redirects=None,
            timeout=None):
        """ Send an OPTIONS request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the OPTIONS request to

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``headers`` a dictionary of headers to use with the request
        """
        session = self._cache.switch(alias)
        redir = True if allow_redirects is None else allow_redirects
        response = self._options_request(session, uri, headers, redir, timeout)
        logger.info(
            'Options Request using : alias=%s, uri=%s, headers=%s, allow_redirects=%s ' %
            (alias, uri, headers, redir))

        return response

    def options(
            self,
            alias,
            uri,
            headers=None,
            allow_redirects=None,
            timeout=None):
        """ **Deprecated- See Options Request now**

        Send an OPTIONS request on the session object found using the
        given `alias`

        ``alias`` that will be used to identify the Session object in the cache

        ``uri`` to send the OPTIONS request to

        ``allow_redirects`` Boolean. Set to True if POST/PUT/DELETE redirect following is allowed.

        ``headers`` a dictionary of headers to use with the request
        """
        logger.warn("Deprecation Warning: Use Options Request in the future")
        session = self._cache.switch(alias)
        redir = True if allow_redirects is None else allow_redirects
        response = self._options_request(session, uri, headers, redir, timeout)

        return response

    def _get_request(
            self,
            session,
            uri,
            params,
            headers,
            json,
            allow_redirects,
            timeout):
        self._capture_output()

        resp = session.get(self._get_url(session, uri),
                           headers=headers,
                           json=json,
                           params=self._utf8_urlencode(params),
                           allow_redirects=allow_redirects,
                           timeout=self._get_timeout(timeout),
                           cookies=self.cookies,
                           verify=self.verify)

        self._print_debug()
        # Store the last session object
        session.last_resp = resp

        return resp

    def _body_request(
            self,
            method_name,
            session,
            uri,
            data,
            json,
            params,
            files,
            headers,
            allow_redirects,
            timeout):
        self._capture_output()

        method = getattr(session, method_name)
        resp = method(self._get_url(session, uri),
                      data=data,
                      json=json,
                      params=self._utf8_urlencode(params),
                      files=files,
                      headers=headers,
                      allow_redirects=allow_redirects,
                      timeout=self._get_timeout(timeout),
                      cookies=self.cookies,
                      verify=self.verify)

        self._print_debug()

        # Store the last session object
        session.last_resp = resp

        self.builtin.log(method_name + ' response: ' + resp.text, 'DEBUG')

        return resp

    def _delete_request(
            self,
            session,
            uri,
            data,
            json,
            params,
            headers,
            allow_redirects,
            timeout):
        self._capture_output()

        resp = session.delete(self._get_url(session, uri),
                              data=data,
                              json=json,
                              params=self._utf8_urlencode(params),
                              headers=headers,
                              allow_redirects=allow_redirects,
                              timeout=self._get_timeout(timeout),
                              cookies=self.cookies,
                              verify=self.verify)

        self._print_debug()

        # Store the last session object
        session.last_resp = resp

        return resp

    def _head_request(self, session, uri, headers, allow_redirects, timeout):
        self._capture_output()

        resp = session.head(self._get_url(session, uri),
                            headers=headers,
                            allow_redirects=allow_redirects,
                            timeout=self._get_timeout(timeout),
                            cookies=self.cookies,
                            verify=self.verify)

        self._print_debug()

        # Store the last session object
        session.last_resp = resp

        return resp

    def _options_request(
            self,
            session,
            uri,
            headers,
            allow_redirects,
            timeout):
        self._capture_output()

        resp = session.options(self._get_url(session, uri),
                               headers=headers,
                               cookies=self.cookies,
                               allow_redirects=allow_redirects,
                               timeout=self._get_timeout(timeout),
                               verify=self.verify)

        self._print_debug()

        # Store the last session object
        session.last_resp = resp

        return resp

    def _get_url(self, session, uri):
        """
        Helper method to get the full url
        """
        url = session.url
        if uri:
            slash = '' if uri.startswith('/') else '/'
            url = "%s%s%s" % (session.url, slash, uri)
        return url

    def _get_timeout(self, timeout):
        return float(timeout) if timeout is not None else self.timeout

    def _capture_output(self):
        if self.debug >= 1:
            self.http_log = WritableObject()
            sys.stdout = self.http_log

    def _print_debug(self):
        if self.debug >= 1:
            sys.stdout = sys.__stdout__  # Restore stdout
            if PY3:
                debug_info = ''.join(
                    self.http_log.content).replace(
                    '\\r',
                    '').replace(
                    '\'',
                    '')
            else:
                debug_info = ''.join(
                    self.http_log.content).replace(
                    '\\r',
                    '').decode('string_escape').replace(
                    '\'',
                    '')

            # Remove empty lines
            debug_info = "\n".join(
                [ll.rstrip() for ll in debug_info.splitlines() if ll.strip()])
            self.builtin.log(debug_info, 'DEBUG')

    def _json_pretty_print(self, content):
        """
        Pretty print a JSON object

        ``content``  JSON object to pretty print
        """
        temp = json.loads(content)
        return json.dumps(
            temp,
            sort_keys=True,
            indent=4,
            separators=(
                ',',
                ': '))

    def _utf8_urlencode(self, data):

        if self._is_string_type(data):
            return data.encode('utf-8')

        if not isinstance(data, dict):
            return data

        utf8_data = {}
        for k, v in data.items():
            if self._is_string_type(v):
                v = v.encode('utf-8')
            utf8_data[k] = v
        return urlencode(utf8_data)

    def _format_data_according_to_header(self, session, data, headers):
        headers = self._merge_headers(session, headers)

        if data is not None and headers is not None and 'Content-Type' in headers and not self._is_json(data):
            if headers['Content-Type'].find("application/json") != -1:
                if not isinstance(data, types.GeneratorType):
                    data = json.dumps(data)
            elif headers['Content-Type'].find("application/x-www-form-urlencoded") != -1:
                data = self._utf8_urlencode(data)
        else:
            data = self._utf8_urlencode(data)

        return data

    def _format_data_to_log_string_according_to_header(self, data, headers):
        dataStr = "<empty>"
        if data is not None and headers is not None and 'Content-Type' in headers:
            if (headers['Content-Type'].find("application/json") != -1) or \
                    (headers['Content-Type'].find("application/x-www-form-urlencoded") != -1):
                if isinstance(data, bytes):
                    dataStr = data.decode('utf-8')
                else:
                    dataStr = data
            else:
                dataStr = "<" + headers['Content-Type'] + ">"

        return dataStr

    @staticmethod
    def _merge_headers(session, headers):
        if headers is None:
            headers = {}
        else:
            headers = headers.copy()

        headers.update(session.headers)

        return headers

    @staticmethod
    def _is_json(data):
        try:
            json.loads(data)
        except (TypeError, ValueError):
            return False
        return True

    @staticmethod
    def _is_string_type(data):
        if PY3 and isinstance(data, str):
            return True
        elif not PY3 and isinstance(data, unicode):
            return True
        return False



# -*- coding:utf-8 -*-
from SCLibrary.base import keyword
from functools import reduce


class SelectKeywords(object):
    @keyword("SELECT ${properties} FROM ${array:[^(WHERE)]}")
    def select_array(self, properties, array):
        """ 返回符合条件的数组 """
        result = []
        if properties == '*':
            return array
        else:
            properties_arr = list(
                map(lambda x: x.strip(), properties.split(',')))
            for item in array:
                prop = {}
                for key in properties_arr:
                    if key in item:
                        prop[key] = item[key]
                    else:
                        prop[key] = None
                result.append(prop)
            return result

    @keyword("SELECTONE ${properties} FROM ${array:[^(WHERE)]}")
    def select_one_array(self, properties, array):
        """ 返回符合条件的第一条数据 """
        return self.select_array(properties, array)[0]

    def condition_parser(self, array, keywords):
        keyword = keywords.pop()

        def intersperse(array, item):
            result = [item] * (len(array) * 2 - 1)
            result[0::2] = array
            return result

        def flatmap(array):
            return reduce(list.__add__, array, [])

        new_array = flatmap(map(lambda item: intersperse(
            item, keyword), map(lambda item: item.split(keyword) if item not in ['!=', '>=', '<='] else [item], array)))
        if len(keywords) == 0:
            return list(filter(lambda item: item != '', new_array))
        else:
            return self.condition_parser(new_array, keywords)

    @keyword("SELECT ${properties} FROM ${array} WHERE ${conditions}")
    def select_array_where(self, properties, array, conditions):
        """ 返回符合条件的数组 """
        items = self.select_array(properties, array)
        keywords = ['>', '<', '=', 'AND', 'and', 'OR', 'or', '!=', '>=', '<=']
        condition_array = self.condition_parser(conditions.split(), keywords)
        operater = {
            '>': lambda x, y: str(x) > str(y),
            '<': lambda x, y: str(x) < str(y),
            '=': lambda x, y: str(x) == str(y),
            '!=': lambda x, y: str(x) != str(y),
            '>=': lambda x, y: str(x) >= str(y),
            '<=': lambda x, y: str(x) <= str(y),
        }

        def fn(item):
            result = True
            mode = 0
            name = None
            op = None
            logic = None
            for condition in condition_array:
                if condition in ['AND', 'and', 'OR', 'or']:
                    mode = 0
                    logic = condition
                elif mode == 0:
                    name = condition
                    mode += 1
                elif mode == 1:
                    op = condition
                    mode += 1
                elif mode == 2:
                    if logic in ['OR', 'or']:
                        if name in item:
                            result |= operater[op](item[name], condition)
                        else:
                            result |= False
                    else:
                        if name in item:
                            result &= operater[op](item[name], condition)
                        else:
                            result &= False
                    mode = 0
                    name = None
                    logic = None
                    op = None

            return result

        return list(filter(fn, items))

    @keyword("SELECTONE ${properties} FROM ${array} WHERE ${conditions}")
    def select_one_array_where(self, properties, array, conditions):
        """ 返回符合条件的第一条数据 """
        return self.select_array_where(properties, array, conditions)[0]




#!/usr/bin/env python

"""Setup script for Robot's SCLibrary distributions"""

import re
from os.path import abspath, dirname, join
from setuptools import setup, find_packages

CURDIR = dirname(abspath(__file__))

with open(join(CURDIR, 'src', 'SCLibrary', '__init__.py')) as f:
    VERSION = re.search("\n__version__ = '(.*)'", f.read()).group(1)
with open(join(CURDIR, 'README.md')) as f:
    DESCRIPTION = f.read()
with open(join(CURDIR, 'requirements.txt')) as f:
    REQUIREMENTS = f.read().splitlines()

def main():
    setup(name             = 'rf-sclibrary',
          version          = VERSION,
          description      = 'SC utility library for Robot Framework',
          long_description = DESCRIPTION,
          author           = 'YoKey',
          author_email     = 'yokeyword@gmail.com',
          url              = 'https://github.com/',
          package_dir      = { '' : 'src'},
          packages         = find_packages('src'),
          install_requires = REQUIREMENTS
          )


if __name__ == "__main__":
    main()


# -*- coding:utf-8 -*-
import re

class LogListener(object):
    ROBOT_LISTENER_API_VERSION = 2

    # def start_suite(self, name, args):
    # print "Starting Suite : " + args['source']

    # def start_test(self, name, args):
    # print "Starting test : " + name
    # if args['template']:
    #     print 'Template is : ' + args['template']

    # def end_test(self, name, args):
    # print "Ending test:  " + args['longname']
    # print "Test Result is : " + args['status']
    # print "Test Time is: " + str(args['elapsedtime'])

    def __init__(self):
        self.pre_log = ''

    def log_message(self, message):
        if  message['message'] == self.pre_log and message['level']=='INFO' and re.search(r'^\${.+}\s=\s', self.pre_log, flags=0) is not None:
            return

        if(message['message'].startswith('INFO :')):
            print("\n" + message['message'])
            return

        if message['level'] == 'FAIL':
            print("\n\033[31m" + message['level'] + " : " + message['message'] + '\033[0m')
        else:
            print("\n" + message['level'] + " : " + message['message'])
        self.pre_log =  message['message']


# coding=utf-8
import time
import os
import socket
import struct
import random
from threading import Thread
import re
import sys
import urllib

from kazoo.protocol.states import KazooState
from kazoo.client import KazooClient


class Registry(object):
    """
    所有注册过的服务端将在这里
    interface=com.ofpay.demo.DemoService
    location = ip:port/url 比如 172.19.20.111:38080/com.ofpay.demo.DemoService2
    providername = servicename|version|group
    dict 格式为{interface:{providername:{ip+port:service_url}}}

    """
    _service_provides = {}

    def _do_event(self, event):
        """
        protect方法，处理回调，留给子类实现
        :param event:
        :return:
        """
        pass

    def subscribe(self, interface, **kwargs):
        """
        监听注册中心的服务上下线
        :param provide_name: 类似com.ofpay.demo.api.UserProvider这样的服务名
        :param kwargs: version , group
        :return: 无返回
        """
        pass

    def get_provides(self, interface, **kwargs):
        """
        获取已经注册的服务URL对象
        :param interface: com.ofpay.demo.api.UserProvider
        :param default:
        :return: 返回一个dict的服务集合
        """
        group = kwargs.get('group', '')
        version = kwargs.get('version', '')
        key = self._to_key(interface, version, group)
        second = self._service_provides.get(interface, {})
        return second.get(key, {})

    def event_listener(self, event):
        """
        node provides上下线的监听回调函数
        :param event:
        :return:
        """
        self._do_event(event)

    def _to_key(self, interface, versioin, group):
        """
        计算存放在内存中的服务的key，以接口、版本、分组计算
        :param interface: 接口 类似com.ofpay.demo.DemoProvider
        :param versioin: 版本 1.0
        :param group:  分组 product
        :return: key 字符串
        """
        return '{0}|{1}|{2}'.format(interface, versioin, group)


class ZookeeperRegistry(Registry):
    _connect_state = 'UNCONNECT'

    def __init__(self, zk_hosts):
        self.__zk = KazooClient(hosts=zk_hosts)
        self.__zk.add_listener(self.__state_listener)
        self.__zk.start()
        self.__zk.add_auth("digest","read:souche-read")

    def __state_listener(self, state):
        if state == KazooState.LOST:
            # Register somewhere that the session was lost
            self._connect_state = state
        elif state == KazooState.SUSPENDED:
            # Handle being disconnected from Zookeeper
            # print 'disconnect from zookeeper'
            self._connect_state = state
        else:
            # Handle being connected/reconnected to Zookeeper
            # print 'connected'
            self._connect_state = state

    def _do_event(self, event):
        # event.path 是类似/dubbo/com.ofpay.demo.api.UserProvider/providers 这样的
        # 如果要删除，必须先把/dubbo/和最后的/providers去掉
        # 将zookeeper中查询到的服务节点列表加入到一个dict中
        # zookeeper中保持的节点url类似如下
        provide_name = event.path[7:event.path.rfind('/')]
        if event.state == 'CONNECTED':
            children = self.__zk.get_children(event.path, watch=self.event_listener)
            self._compare_swap_nodes(provide_name, self.__unquote(children))
        if event.state == 'DELETED':
            children = self.__zk.get_children(event.path, watch=self.event_listener)
            self._compare_swap_nodes(provide_name, self.__unquote(children))


    def subscribe(self, interface, **kwargs):
        """
        监听注册中心的服务上下线
        :param interface: 类似com.ofpay.demo.api.UserProvider这样的服务名
        :return: 无返回
        """
        version = kwargs.get('version', '')
        group = kwargs.get('group', '')
        children = self.__zk.get_children('{0}/{1}/{2}'.format('dubbo', interface, 'providers'),watch=self.event_listener)
        provide = random.choice(children)
        provide = urllib.parse.unquote(provide)
        ip = re.findall(r"\d{0,3}\.\d{0,3}\.\d{0,3}\.\d{0,3}:\d{2,5}", provide)
        return  ip